@@ -14,31 +14,12 @@
 #include <config.h>
 #endif
 
-#include "soup-ntlm.h"
-
 #include <ctype.h>
+#include <stdlib.h>
 #include <string.h>
 
-/* Base64 */
-static int base64_encode_close    (unsigned char       *in, 
-				   int                  inlen, 
-				   gboolean             break_lines, 
-				   unsigned char       *out, 
-				   int                 *state, 
-				   int                 *save);
-
-static int base64_encode_step     (unsigned char       *in, 
-				   int                  len, 
-				   gboolean             break_lines, 
-				   unsigned char       *out, 
-				   int                 *state, 
-				   int                 *save);
-
-static int base64_decode_step     (unsigned char       *in, 
-				   int                  len, 
-				   unsigned char       *out, 
-				   int                 *state,
-				   unsigned int        *save);
+#include "soup-ntlm.h"
+#include "soup-misc.h"
 
 /* MD4 */
 static void md4sum                (const unsigned char *in, 
@@ -63,7 +44,7 @@ static void calc_response         (const guchar        *key,
 			  "\x00\x00\x00\x00\x00"
 
 void
-soup_ntlm_lanmanager_hash (const char *password, char hash[21])
+soup_ntlm_lanmanager_hash (const char *password, guchar hash[21])
 {
 	guchar lm_password [15];
 	DES_KS ks;
@@ -85,7 +66,7 @@ soup_ntlm_lanmanager_hash (const char *password, char hash[21])
 }
 
 void
-soup_ntlm_nt_hash (const char *password, char hash[21])
+soup_ntlm_nt_hash (const char *password, guchar hash[21])
 {
 	unsigned char *buf, *p;
 
@@ -109,16 +90,9 @@ typedef struct {
 	guchar  zero_pad[2];
 } NTLMString;
 
-#define NTLM_REQUEST_HEADER "NTLMSSP\x00\x01\x00\x00\x00\xB2\x03\x00\x00"
-
-typedef struct {
-	guchar     header[16];
-	NTLMString domain;
-	NTLMString host;
-} NTLMRequest;
-
-#define NTLM_CHALLENGE_NONCE_OFFSET 24
-#define NTLM_CHALLENGE_NONCE_LENGTH 8
+#define NTLM_CHALLENGE_NONCE_OFFSET         24
+#define NTLM_CHALLENGE_NONCE_LENGTH          8
+#define NTLM_CHALLENGE_DOMAIN_STRING_OFFSET 12
 
 #define NTLM_RESPONSE_HEADER "NTLMSSP\x00\x03\x00\x00\x00"
 #define NTLM_RESPONSE_FLAGS "\x82\x01"
@@ -141,8 +115,10 @@ typedef struct {
 
 #if G_BYTE_ORDER == G_BIG_ENDIAN
 #define LE16(x) (((x & 0xFF) << 8) | ((x >> 8) & 0xFF))
+#define UNLE16(x) LE16(x)
 #else
 #define LE16(x) x
+#define UNLE16(x) x
 #endif
 
 static void
@@ -154,91 +130,92 @@ ntlm_set_string (NTLMString *string, int *offset, int len)
 }
 
 char *
-soup_ntlm_request (const char *host, const char *domain)
+soup_ntlm_request (void)
 {
-	NTLMRequest req;
-	unsigned char *out, *p;
-	int hlen = strlen (host), dlen = strlen (domain), offset;
-	int state, save;
-
-	memset (&req, 0, sizeof (req));
-	memcpy (req.header, NTLM_REQUEST_HEADER, sizeof (req.header));
-
-	offset = sizeof (req);
-	ntlm_set_string (&req.host, &offset, hlen);
-	ntlm_set_string (&req.domain, &offset, dlen);
-
-	out = g_malloc (((offset + 3) * 4) / 3 + 6);
-	strncpy (out, "NTLM ", 5);
-	p = out + 5;
-
-	state = save = 0;
-	p += base64_encode_step ((guchar *) &req, 
-				 sizeof (req), 
-				 FALSE, 
-				 p, 
-				 &state, 
-				 &save);
-	p += base64_encode_step ((guchar *) host, 
-				 hlen, 
-				 FALSE, 
-				 p, 
-				 &state, 
-				 &save);
-	p += base64_encode_close ((guchar *) domain, 
-				  dlen, 
-				  FALSE, 
-				  p, 
-				  &state, 
-				  &save);
-	*p = '\0';
-
-	return out;
+	return g_strdup ("NTLM TlRMTVNTUAABAAAABoIAAAAAAAAAAAAAAAAAAAAAAAAAAAAAMAAAAAAAAAAwAAAA");
 }
 
-char *
-soup_ntlm_response (const char *challenge, 
-		    const char *user,
-		    const char *lm_hash, 
-		    const char *nt_hash,
-		    const char *host, 
-		    const char *domain)
+gboolean
+soup_ntlm_parse_challenge (const char *challenge,
+			   char      **nonce,
+			   char      **default_domain)
 {
-	int hlen = strlen (host), dlen = strlen (domain);
-	int ulen = strlen (user), offset, decodelen;
-	guchar lm_resp[24], nt_resp[24], *nonce;
-	NTLMResponse resp;
+	int clen, decodelen;
+	NTLMString domain;
 	char *chall;
-	unsigned char *out, *p;
 	int state, save;
 
 	if (strncmp (challenge, "NTLM ", 5) != 0)
-		return NULL;
+		return FALSE;
 
-	decodelen = strlen (challenge);
+	decodelen = strlen (challenge) - 5;
 	chall = g_malloc (decodelen);
 
 	state = save = 0;
-	base64_decode_step ((guchar *) challenge + 5, 
-			    decodelen,
-			    chall, 
-			    &state, 
-			    &save);
+	clen = soup_base64_decode_step ((guchar *) challenge + 5, 
+					decodelen,
+					chall, 
+					&state, 
+					&save);
 
-	nonce = chall + NTLM_CHALLENGE_NONCE_OFFSET;
-	nonce [NTLM_CHALLENGE_NONCE_LENGTH] = '\0';
+	if (clen < NTLM_CHALLENGE_DOMAIN_STRING_OFFSET ||
+	    clen < NTLM_CHALLENGE_NONCE_OFFSET + NTLM_CHALLENGE_NONCE_LENGTH) {
+		g_free (chall);
+		return FALSE;
+	}
 
-	calc_response (lm_hash, nonce, lm_resp);
-	calc_response (nt_hash, nonce, nt_resp);
-	g_free (chall);
+	if (default_domain) {
+		memcpy (&domain, chall + NTLM_CHALLENGE_DOMAIN_STRING_OFFSET, sizeof (domain));
+		domain.length = UNLE16 (domain.length);
+		domain.offset = UNLE16 (domain.offset);
+
+		if (clen < domain.length + domain.offset) {
+			g_free (chall);
+			return FALSE;
+		}
+
+		*default_domain = g_strndup (chall + domain.offset, domain.length);
+	}
+
+	if (nonce) {
+		*nonce = g_strndup (chall + NTLM_CHALLENGE_NONCE_OFFSET,
+				    NTLM_CHALLENGE_NONCE_LENGTH);
+	}
+
+	return TRUE;
+}
+
+char *
+soup_ntlm_response (const char *nonce, 
+		    const char *user,
+		    const char *password, 
+		    const char *host, 
+		    const char *domain)
+{
+	int hlen, dlen, ulen, offset;
+	guchar hash[21], lm_resp[24], nt_resp[24];
+	NTLMResponse resp;
+	unsigned char *out, *p;
+	int state, save;
+
+	soup_ntlm_lanmanager_hash (password, hash);
+	calc_response (hash, nonce, lm_resp);
+	soup_ntlm_nt_hash (password, hash);
+	calc_response (hash, nonce, nt_resp);
 
 	memset (&resp, 0, sizeof (resp));
 	memcpy (resp.header, NTLM_RESPONSE_HEADER, sizeof (resp.header));
 	memcpy (resp.flags, NTLM_RESPONSE_FLAGS, sizeof (resp.flags));
 
 	offset = sizeof (resp);
+
+	dlen = strlen (domain);
 	ntlm_set_string (&resp.domain, &offset, dlen);
+	ulen = strlen (user);
 	ntlm_set_string (&resp.user, &offset, ulen);
+	if (!host)
+		host = "UNKNOWN";
+	hlen = strlen (host);
 	ntlm_set_string (&resp.host, &offset, hlen);
 	ntlm_set_string (&resp.lm_resp, &offset, sizeof (lm_resp));
 	ntlm_set_string (&resp.nt_resp, &offset, sizeof (nt_resp));
@@ -249,42 +226,43 @@ soup_ntlm_response (const char *challenge,
 
 	state = save = 0;
 
-	p += base64_encode_step ((guchar *) &resp, 
-				 sizeof (resp), 
-				 FALSE, 
-				 p, 
-				 &state, 
-				 &save);
-	p += base64_encode_step ((guchar *) domain, 
-				 dlen, 
-				 FALSE, 
-				 p, 
-				 &state, 
-				 &save);
-	p += base64_encode_step ((guchar *) user, 
-				 ulen, 
-				 FALSE, 
-				 p, 
-				 &state, 
-				 &save);
-	p += base64_encode_step ((guchar *) host, 
-				 hlen, 
-				 FALSE, 
-				 p,
-				 &state, 
-				 &save);
-	p += base64_encode_step (lm_resp, 
-				 sizeof (lm_resp), 
-				 FALSE, 
-				 p, 
-				 &state, 
-				 &save);
-	p += base64_encode_close (nt_resp, 
-				  sizeof (nt_resp), 
-				  FALSE, 
-				  p, 
-				  &state, 
-				  &save);
+	p += soup_base64_encode_step ((guchar *) &resp, 
+				      sizeof (resp), 
+				      FALSE, 
+				      p, 
+				      &state, 
+				      &save);
+	p += soup_base64_encode_step ((guchar *) domain, 
+				      dlen, 
+				      FALSE, 
+				      p, 
+				      &state, 
+				      &save);
+	p += soup_base64_encode_step ((guchar *) user, 
+				      ulen, 
+				      FALSE, 
+				      p, 
+				      &state, 
+				      &save);
+	p += soup_base64_encode_step ((guchar *) host, 
+				      hlen, 
+				      FALSE, 
+				      p,
+				      &state, 
+				      &save);
+	p += soup_base64_encode_step (lm_resp, 
+				      sizeof (lm_resp), 
+				      FALSE, 
+				      p, 
+				      &state, 
+				      &save);
+
+	p += soup_base64_encode_close (nt_resp, 
+				       sizeof (nt_resp), 
+				       FALSE, 
+				       p, 
+				       &state, 
+				       &save);
 	*p = '\0';
 
 	return out;
@@ -334,240 +312,6 @@ calc_response (const guchar *key, const guchar *plaintext, guchar *results)
         des (ks, results + 16);
 }
 
-/* Base64 utils (straight from camel-mime-utils.c) */
-#define d(x)
-
-static char *base64_alphabet =
-"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-
-static unsigned char camel_mime_base64_rank[256] = {
-	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
-	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
-	255,255,255,255,255,255,255,255,255,255,255, 62,255,255,255, 63,
-	 52, 53, 54, 55, 56, 57, 58, 59, 60, 61,255,255,255,  0,255,255,
-	255,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
-	 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25,255,255,255,255,255,
-	255, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
-	 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51,255,255,255,255,255,
-	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
-	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
-	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
-	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
-	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
-	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
-	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
-	255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,
-};
-
-/* 
- * call this when finished encoding everything, to
- * flush off the last little bit 
- */
-static int
-base64_encode_close (unsigned char *in, 
-		     int            inlen, 
-		     gboolean       break_lines, 
-		     unsigned char *out, 
-		     int           *state, 
-		     int           *save)
-{
-	int c1, c2;
-	unsigned char *outptr = out;
-
-	if (inlen > 0)
-		outptr += base64_encode_step (in, 
-					      inlen, 
-					      break_lines, 
-					      outptr, 
-					      state, 
-					      save);
-
-	c1 = ((unsigned char *) save) [1];
-	c2 = ((unsigned char *) save) [2];
-	
-	d(printf("mode = %d\nc1 = %c\nc2 = %c\n",
-		 (int)((char *) save) [0],
-		 (int)((char *) save) [1],
-		 (int)((char *) save) [2]));
-
-	switch (((char *) save) [0]) {
-	case 2:
-		outptr [2] = base64_alphabet[ ( (c2 &0x0f) << 2 ) ];
-		g_assert (outptr [2] != 0);
-		goto skip;
-	case 1:
-		outptr[2] = '=';
-	skip:
-		outptr [0] = base64_alphabet [ c1 >> 2 ];
-		outptr [1] = base64_alphabet [ c2 >> 4 | ( (c1&0x3) << 4 )];
-		outptr [3] = '=';
-		outptr += 4;
-		break;
-	}
-	if (break_lines)
-		*outptr++ = '\n';
-
-	*save = 0;
-	*state = 0;
-
-	return outptr-out;
-}
-
-/*
- * performs an 'encode step', only encodes blocks of 3 characters to the
- * output at a time, saves left-over state in state and save (initialise to
- * 0 on first invocation).
- */
-static int
-base64_encode_step(unsigned char *in, 
-		   int            len, 
-		   gboolean       break_lines, 
-		   unsigned char *out, 
-		   int           *state, 
-		   int           *save)
-{
-	register unsigned char *inptr, *outptr;
-
-	if (len <= 0)
-		return 0;
-
-	inptr = in;
-	outptr = out;
-
-	d (printf ("we have %d chars, and %d saved chars\n", 
-		   len, 
-		   ((char *) save) [0]));
-
-	if (len + ((char *) save) [0] > 2) {
-		unsigned char *inend = in+len-2;
-		register int c1, c2, c3;
-		register int already;
-
-		already = *state;
-
-		switch (((char *) save) [0]) {
-		case 1:	c1 = ((unsigned char *) save) [1]; goto skip1;
-		case 2:	c1 = ((unsigned char *) save) [1];
-			c2 = ((unsigned char *) save) [2]; goto skip2;
-		}
-		
-		/* 
-		 * yes, we jump into the loop, no i'm not going to change it, 
-		 * it's beautiful! 
-		 */
-		while (inptr < inend) {
-			c1 = *inptr++;
-		skip1:
-			c2 = *inptr++;
-		skip2:
-			c3 = *inptr++;
-			*outptr++ = base64_alphabet [ c1 >> 2 ];
-			*outptr++ = base64_alphabet [ c2 >> 4 | 
-						      ((c1&0x3) << 4) ];
-			*outptr++ = base64_alphabet [ ((c2 &0x0f) << 2) | 
-						      (c3 >> 6) ];
-			*outptr++ = base64_alphabet [ c3 & 0x3f ];
-			/* this is a bit ugly ... */
-			if (break_lines && (++already)>=19) {
-				*outptr++='\n';
-				already = 0;
-			}
-		}
-
-		((char *)save)[0] = 0;
-		len = 2-(inptr-inend);
-		*state = already;
-	}
-
-	d(printf("state = %d, len = %d\n",
-		 (int)((char *)save)[0],
-		 len));
-
-	if (len>0) {
-		register char *saveout;
-
-		/* points to the slot for the next char to save */
-		saveout = & (((char *)save)[1]) + ((char *)save)[0];
-
-		/* len can only be 0 1 or 2 */
-		switch(len) {
-		case 2:	*saveout++ = *inptr++;
-		case 1:	*saveout++ = *inptr++;
-		}
-		((char *)save)[0]+=len;
-	}
-
-	d(printf("mode = %d\nc1 = %c\nc2 = %c\n",
-		 (int)((char *)save)[0],
-		 (int)((char *)save)[1],
-		 (int)((char *)save)[2]));
-
-	return outptr-out;
-}
-
-
-/**
- * base64_decode_step: decode a chunk of base64 encoded data
- * @in: input stream
- * @len: max length of data to decode
- * @out: output stream
- * @state: holds the number of bits that are stored in @save
- * @save: leftover bits that have not yet been decoded
- *
- * Decodes a chunk of base64 encoded data
- **/
-static int
-base64_decode_step (unsigned char *in, 
-		    int            len, 
-		    unsigned char *out, 
-		    int           *state, 
-		    unsigned int  *save)
-{
-	register unsigned char *inptr, *outptr;
-	unsigned char *inend, c;
-	register unsigned int v;
-	int i;
-
-	inend = in+len;
-	outptr = out;
-
-	/* convert 4 base64 bytes to 3 normal bytes */
-	v=*save;
-	i=*state;
-	inptr = in;
-	while (inptr < inend) {
-		c = camel_mime_base64_rank [*inptr++];
-		if (c != 0xff) {
-			v = (v<<6) | c;
-			i++;
-			if (i==4) {
-				*outptr++ = v>>16;
-				*outptr++ = v>>8;
-				*outptr++ = v;
-				i=0;
-			}
-		}
-	}
-
-	*save = v;
-	*state = i;
-
-	/* quick scan back for '=' on the end somewhere */
-	/* fortunately we can drop 1 output char for each trailing = (upto 2) */
-	i=2;
-	while (inptr > in && i) {
-		inptr--;
-		if (camel_mime_base64_rank [*inptr] != 0xff) {
-			if (*inptr == '=')
-				outptr--;
-			i--;
-		}
-	}
-
-	/* if i!= 0 then there is a truncation error! */
-	return outptr - out;
-}
-
 
 /* 
  * MD4 encoder. (The one everyone else uses is not GPL-compatible;
@@ -694,134 +438,134 @@ md4sum (const unsigned char *in, int nbytes, unsigned char digest[16])
 
 /* Public domain DES implementation from Phil Karn */
 static unsigned long Spbox[8][64] = {
-0x01010400,0x00000000,0x00010000,0x01010404,
-0x01010004,0x00010404,0x00000004,0x00010000,
-0x00000400,0x01010400,0x01010404,0x00000400,
-0x01000404,0x01010004,0x01000000,0x00000004,
-0x00000404,0x01000400,0x01000400,0x00010400,
-0x00010400,0x01010000,0x01010000,0x01000404,
-0x00010004,0x01000004,0x01000004,0x00010004,
-0x00000000,0x00000404,0x00010404,0x01000000,
-0x00010000,0x01010404,0x00000004,0x01010000,
-0x01010400,0x01000000,0x01000000,0x00000400,
-0x01010004,0x00010000,0x00010400,0x01000004,
-0x00000400,0x00000004,0x01000404,0x00010404,
-0x01010404,0x00010004,0x01010000,0x01000404,
-0x01000004,0x00000404,0x00010404,0x01010400,
-0x00000404,0x01000400,0x01000400,0x00000000,
-0x00010004,0x00010400,0x00000000,0x01010004,
-0x80108020,0x80008000,0x00008000,0x00108020,
-0x00100000,0x00000020,0x80100020,0x80008020,
-0x80000020,0x80108020,0x80108000,0x80000000,
-0x80008000,0x00100000,0x00000020,0x80100020,
-0x00108000,0x00100020,0x80008020,0x00000000,
-0x80000000,0x00008000,0x00108020,0x80100000,
-0x00100020,0x80000020,0x00000000,0x00108000,
-0x00008020,0x80108000,0x80100000,0x00008020,
-0x00000000,0x00108020,0x80100020,0x00100000,
-0x80008020,0x80100000,0x80108000,0x00008000,
-0x80100000,0x80008000,0x00000020,0x80108020,
-0x00108020,0x00000020,0x00008000,0x80000000,
-0x00008020,0x80108000,0x00100000,0x80000020,
-0x00100020,0x80008020,0x80000020,0x00100020,
-0x00108000,0x00000000,0x80008000,0x00008020,
-0x80000000,0x80100020,0x80108020,0x00108000,
-0x00000208,0x08020200,0x00000000,0x08020008,
-0x08000200,0x00000000,0x00020208,0x08000200,
-0x00020008,0x08000008,0x08000008,0x00020000,
-0x08020208,0x00020008,0x08020000,0x00000208,
-0x08000000,0x00000008,0x08020200,0x00000200,
-0x00020200,0x08020000,0x08020008,0x00020208,
-0x08000208,0x00020200,0x00020000,0x08000208,
-0x00000008,0x08020208,0x00000200,0x08000000,
-0x08020200,0x08000000,0x00020008,0x00000208,
-0x00020000,0x08020200,0x08000200,0x00000000,
-0x00000200,0x00020008,0x08020208,0x08000200,
-0x08000008,0x00000200,0x00000000,0x08020008,
-0x08000208,0x00020000,0x08000000,0x08020208,
-0x00000008,0x00020208,0x00020200,0x08000008,
-0x08020000,0x08000208,0x00000208,0x08020000,
-0x00020208,0x00000008,0x08020008,0x00020200,
-0x00802001,0x00002081,0x00002081,0x00000080,
-0x00802080,0x00800081,0x00800001,0x00002001,
-0x00000000,0x00802000,0x00802000,0x00802081,
-0x00000081,0x00000000,0x00800080,0x00800001,
-0x00000001,0x00002000,0x00800000,0x00802001,
-0x00000080,0x00800000,0x00002001,0x00002080,
-0x00800081,0x00000001,0x00002080,0x00800080,
-0x00002000,0x00802080,0x00802081,0x00000081,
-0x00800080,0x00800001,0x00802000,0x00802081,
-0x00000081,0x00000000,0x00000000,0x00802000,
-0x00002080,0x00800080,0x00800081,0x00000001,
-0x00802001,0x00002081,0x00002081,0x00000080,
-0x00802081,0x00000081,0x00000001,0x00002000,
-0x00800001,0x00002001,0x00802080,0x00800081,
-0x00002001,0x00002080,0x00800000,0x00802001,
-0x00000080,0x00800000,0x00002000,0x00802080,
-0x00000100,0x02080100,0x02080000,0x42000100,
-0x00080000,0x00000100,0x40000000,0x02080000,
-0x40080100,0x00080000,0x02000100,0x40080100,
-0x42000100,0x42080000,0x00080100,0x40000000,
-0x02000000,0x40080000,0x40080000,0x00000000,
-0x40000100,0x42080100,0x42080100,0x02000100,
-0x42080000,0x40000100,0x00000000,0x42000000,
-0x02080100,0x02000000,0x42000000,0x00080100,
-0x00080000,0x42000100,0x00000100,0x02000000,
-0x40000000,0x02080000,0x42000100,0x40080100,
-0x02000100,0x40000000,0x42080000,0x02080100,
-0x40080100,0x00000100,0x02000000,0x42080000,
-0x42080100,0x00080100,0x42000000,0x42080100,
-0x02080000,0x00000000,0x40080000,0x42000000,
-0x00080100,0x02000100,0x40000100,0x00080000,
-0x00000000,0x40080000,0x02080100,0x40000100,
-0x20000010,0x20400000,0x00004000,0x20404010,
-0x20400000,0x00000010,0x20404010,0x00400000,
-0x20004000,0x00404010,0x00400000,0x20000010,
-0x00400010,0x20004000,0x20000000,0x00004010,
-0x00000000,0x00400010,0x20004010,0x00004000,
-0x00404000,0x20004010,0x00000010,0x20400010,
-0x20400010,0x00000000,0x00404010,0x20404000,
-0x00004010,0x00404000,0x20404000,0x20000000,
-0x20004000,0x00000010,0x20400010,0x00404000,
-0x20404010,0x00400000,0x00004010,0x20000010,
-0x00400000,0x20004000,0x20000000,0x00004010,
-0x20000010,0x20404010,0x00404000,0x20400000,
-0x00404010,0x20404000,0x00000000,0x20400010,
-0x00000010,0x00004000,0x20400000,0x00404010,
-0x00004000,0x00400010,0x20004010,0x00000000,
-0x20404000,0x20000000,0x00400010,0x20004010,
-0x00200000,0x04200002,0x04000802,0x00000000,
-0x00000800,0x04000802,0x00200802,0x04200800,
-0x04200802,0x00200000,0x00000000,0x04000002,
-0x00000002,0x04000000,0x04200002,0x00000802,
-0x04000800,0x00200802,0x00200002,0x04000800,
-0x04000002,0x04200000,0x04200800,0x00200002,
-0x04200000,0x00000800,0x00000802,0x04200802,
-0x00200800,0x00000002,0x04000000,0x00200800,
-0x04000000,0x00200800,0x00200000,0x04000802,
-0x04000802,0x04200002,0x04200002,0x00000002,
-0x00200002,0x04000000,0x04000800,0x00200000,
-0x04200800,0x00000802,0x00200802,0x04200800,
-0x00000802,0x04000002,0x04200802,0x04200000,
-0x00200800,0x00000000,0x00000002,0x04200802,
-0x00000000,0x00200802,0x04200000,0x00000800,
-0x04000002,0x04000800,0x00000800,0x00200002,
-0x10001040,0x00001000,0x00040000,0x10041040,
-0x10000000,0x10001040,0x00000040,0x10000000,
-0x00040040,0x10040000,0x10041040,0x00041000,
-0x10041000,0x00041040,0x00001000,0x00000040,
-0x10040000,0x10000040,0x10001000,0x00001040,
-0x00041000,0x00040040,0x10040040,0x10041000,
-0x00001040,0x00000000,0x00000000,0x10040040,
-0x10000040,0x10001000,0x00041040,0x00040000,
-0x00041040,0x00040000,0x10041000,0x00001000,
-0x00000040,0x10040040,0x00001000,0x00041040,
-0x10001000,0x00000040,0x10000040,0x10040000,
-0x10040040,0x10000000,0x00040000,0x10001040,
-0x00000000,0x10041040,0x00040040,0x10000040,
-0x10040000,0x10001000,0x10001040,0x00000000,
-0x10041040,0x00041000,0x00041000,0x00001040,
-0x00001040,0x00040040,0x10000000,0x10041000,
+	0x01010400,0x00000000,0x00010000,0x01010404,
+	0x01010004,0x00010404,0x00000004,0x00010000,
+	0x00000400,0x01010400,0x01010404,0x00000400,
+	0x01000404,0x01010004,0x01000000,0x00000004,
+	0x00000404,0x01000400,0x01000400,0x00010400,
+	0x00010400,0x01010000,0x01010000,0x01000404,
+	0x00010004,0x01000004,0x01000004,0x00010004,
+	0x00000000,0x00000404,0x00010404,0x01000000,
+	0x00010000,0x01010404,0x00000004,0x01010000,
+	0x01010400,0x01000000,0x01000000,0x00000400,
+	0x01010004,0x00010000,0x00010400,0x01000004,
+	0x00000400,0x00000004,0x01000404,0x00010404,
+	0x01010404,0x00010004,0x01010000,0x01000404,
+	0x01000004,0x00000404,0x00010404,0x01010400,
+	0x00000404,0x01000400,0x01000400,0x00000000,
+	0x00010004,0x00010400,0x00000000,0x01010004,
+	0x80108020,0x80008000,0x00008000,0x00108020,
+	0x00100000,0x00000020,0x80100020,0x80008020,
+	0x80000020,0x80108020,0x80108000,0x80000000,
+	0x80008000,0x00100000,0x00000020,0x80100020,
+	0x00108000,0x00100020,0x80008020,0x00000000,
+	0x80000000,0x00008000,0x00108020,0x80100000,
+	0x00100020,0x80000020,0x00000000,0x00108000,
+	0x00008020,0x80108000,0x80100000,0x00008020,
+	0x00000000,0x00108020,0x80100020,0x00100000,
+	0x80008020,0x80100000,0x80108000,0x00008000,
+	0x80100000,0x80008000,0x00000020,0x80108020,
+	0x00108020,0x00000020,0x00008000,0x80000000,
+	0x00008020,0x80108000,0x00100000,0x80000020,
+	0x00100020,0x80008020,0x80000020,0x00100020,
+	0x00108000,0x00000000,0x80008000,0x00008020,
+	0x80000000,0x80100020,0x80108020,0x00108000,
+	0x00000208,0x08020200,0x00000000,0x08020008,
+	0x08000200,0x00000000,0x00020208,0x08000200,
+	0x00020008,0x08000008,0x08000008,0x00020000,
+	0x08020208,0x00020008,0x08020000,0x00000208,
+	0x08000000,0x00000008,0x08020200,0x00000200,
+	0x00020200,0x08020000,0x08020008,0x00020208,
+	0x08000208,0x00020200,0x00020000,0x08000208,
+	0x00000008,0x08020208,0x00000200,0x08000000,
+	0x08020200,0x08000000,0x00020008,0x00000208,
+	0x00020000,0x08020200,0x08000200,0x00000000,
+	0x00000200,0x00020008,0x08020208,0x08000200,
+	0x08000008,0x00000200,0x00000000,0x08020008,
+	0x08000208,0x00020000,0x08000000,0x08020208,
+	0x00000008,0x00020208,0x00020200,0x08000008,
+	0x08020000,0x08000208,0x00000208,0x08020000,
+	0x00020208,0x00000008,0x08020008,0x00020200,
+	0x00802001,0x00002081,0x00002081,0x00000080,
+	0x00802080,0x00800081,0x00800001,0x00002001,
+	0x00000000,0x00802000,0x00802000,0x00802081,
+	0x00000081,0x00000000,0x00800080,0x00800001,
+	0x00000001,0x00002000,0x00800000,0x00802001,
+	0x00000080,0x00800000,0x00002001,0x00002080,
+	0x00800081,0x00000001,0x00002080,0x00800080,
+	0x00002000,0x00802080,0x00802081,0x00000081,
+	0x00800080,0x00800001,0x00802000,0x00802081,
+	0x00000081,0x00000000,0x00000000,0x00802000,
+	0x00002080,0x00800080,0x00800081,0x00000001,
+	0x00802001,0x00002081,0x00002081,0x00000080,
+	0x00802081,0x00000081,0x00000001,0x00002000,
+	0x00800001,0x00002001,0x00802080,0x00800081,
+	0x00002001,0x00002080,0x00800000,0x00802001,
+	0x00000080,0x00800000,0x00002000,0x00802080,
+	0x00000100,0x02080100,0x02080000,0x42000100,
+	0x00080000,0x00000100,0x40000000,0x02080000,
+	0x40080100,0x00080000,0x02000100,0x40080100,
+	0x42000100,0x42080000,0x00080100,0x40000000,
+	0x02000000,0x40080000,0x40080000,0x00000000,
+	0x40000100,0x42080100,0x42080100,0x02000100,
+	0x42080000,0x40000100,0x00000000,0x42000000,
+	0x02080100,0x02000000,0x42000000,0x00080100,
+	0x00080000,0x42000100,0x00000100,0x02000000,
+	0x40000000,0x02080000,0x42000100,0x40080100,
+	0x02000100,0x40000000,0x42080000,0x02080100,
+	0x40080100,0x00000100,0x02000000,0x42080000,
+	0x42080100,0x00080100,0x42000000,0x42080100,
+	0x02080000,0x00000000,0x40080000,0x42000000,
+	0x00080100,0x02000100,0x40000100,0x00080000,
+	0x00000000,0x40080000,0x02080100,0x40000100,
+	0x20000010,0x20400000,0x00004000,0x20404010,
+	0x20400000,0x00000010,0x20404010,0x00400000,
+	0x20004000,0x00404010,0x00400000,0x20000010,
+	0x00400010,0x20004000,0x20000000,0x00004010,
+	0x00000000,0x00400010,0x20004010,0x00004000,
+	0x00404000,0x20004010,0x00000010,0x20400010,
+	0x20400010,0x00000000,0x00404010,0x20404000,
+	0x00004010,0x00404000,0x20404000,0x20000000,
+	0x20004000,0x00000010,0x20400010,0x00404000,
+	0x20404010,0x00400000,0x00004010,0x20000010,
+	0x00400000,0x20004000,0x20000000,0x00004010,
+	0x20000010,0x20404010,0x00404000,0x20400000,
+	0x00404010,0x20404000,0x00000000,0x20400010,
+	0x00000010,0x00004000,0x20400000,0x00404010,
+	0x00004000,0x00400010,0x20004010,0x00000000,
+	0x20404000,0x20000000,0x00400010,0x20004010,
+	0x00200000,0x04200002,0x04000802,0x00000000,
+	0x00000800,0x04000802,0x00200802,0x04200800,
+	0x04200802,0x00200000,0x00000000,0x04000002,
+	0x00000002,0x04000000,0x04200002,0x00000802,
+	0x04000800,0x00200802,0x00200002,0x04000800,
+	0x04000002,0x04200000,0x04200800,0x00200002,
+	0x04200000,0x00000800,0x00000802,0x04200802,
+	0x00200800,0x00000002,0x04000000,0x00200800,
+	0x04000000,0x00200800,0x00200000,0x04000802,
+	0x04000802,0x04200002,0x04200002,0x00000002,
+	0x00200002,0x04000000,0x04000800,0x00200000,
+	0x04200800,0x00000802,0x00200802,0x04200800,
+	0x00000802,0x04000002,0x04200802,0x04200000,
+	0x00200800,0x00000000,0x00000002,0x04200802,
+	0x00000000,0x00200802,0x04200000,0x00000800,
+	0x04000002,0x04000800,0x00000800,0x00200002,
+	0x10001040,0x00001000,0x00040000,0x10041040,
+	0x10000000,0x10001040,0x00000040,0x10000000,
+	0x00040040,0x10040000,0x10041040,0x00041000,
+	0x10041000,0x00041040,0x00001000,0x00000040,
+	0x10040000,0x10000040,0x10001000,0x00001040,
+	0x00041000,0x00040040,0x10040040,0x10041000,
+	0x00001040,0x00000000,0x00000000,0x10040040,
+	0x10000040,0x10001000,0x00041040,0x00040000,
+	0x00041040,0x00040000,0x10041000,0x00001000,
+	0x00000040,0x10040040,0x00001000,0x00041040,
+	0x10001000,0x00000040,0x10000040,0x10040000,
+	0x10040040,0x10000000,0x00040000,0x10001040,
+	0x00000000,0x10041040,0x00040040,0x10000040,
+	0x10040000,0x10001000,0x10001040,0x00000000,
+	0x10041040,0x00041000,0x00041000,0x00001040,
+	0x00001040,0x00040040,0x10000000,0x10041000,
 };
 
 #undef F
